@@ -40,7 +40,8 @@
   #:use-module (cv imgproc)
   #:use-module (cv segmentation)
   #:use-module (cv features)
-  
+  #:use-module (cv utils)
+
   #:duplicates (merge-generics
 		replace
 		warn-override-core
@@ -53,7 +54,9 @@
             im-rgba->gray
 	    im-threshold
 	    im-and
+            im-and-channel
 	    im-or
+            im-or-channel
             im-xor
 	    im-complement
             im-range
@@ -70,8 +73,10 @@
             im-normalize
             im-normalize-channel
             im-scrap
+            im-scrap-channel
             im-particles
-            im-particle-clean))
+            im-particle-clean
+            im-composite->rgb))
 
 
 (g-export im-add
@@ -88,37 +93,104 @@
 ;;; Guile-CV additional API
 ;;;
 
-(define (im-rgb->gray-1 c r g b i mini maxi total n-cell)
-  (if (= i n-cell)
-      (list mini maxi (round (/ total n-cell)) n-cell)
-      (let ((k (/ (+ (f32vector-ref r i)
-		     (f32vector-ref g i)
-		     (f32vector-ref b i))
-		  3)))
-	(f32vector-set! c i k)
-	(im-rgb->gray-1 c r g b
-			(+ i 1)
-			(float-round (min mini k) 1)
-			(float-round (max maxi k) 1)
-			(+ total k)
-			n-cell))))
+#;(define (im-rgb->gray image)
+  (match image
+    ((width height n-chan idata)
+     (match idata
+       ((c)
+        image)
+       ((r g b)
+        (receive (c-chan extra)
+            (im-rgb->gray-1 width height r g b)
+          (values (list width height 1 (list c-chan))
+                  extra)))
+       (else
+	(error "Not an RGB (nor a GRAY) image."))))))
 
 (define (im-rgb->gray image)
   (match image
     ((width height n-chan idata)
-     (case n-chan
-       ((1)
-	image)
-       ((3)
-	(match idata
-	  ((r g b)
-	   (let* ((c (im-make-channel width height))
-		  (vals (im-rgb->gray-1 c r g b 0 0 0 0 (* width height))))
-	     (values (list width height 1 (list c))
-		     vals)))))
+     (match idata
+       ((c)
+        image)
+       ((r g b)
+        (list width height 1
+              (list (im-rgb->gray-1 width height r g b))))
        (else
 	(error "Not an RGB (nor a GRAY) image."))))))
 
+(define (im-rgb->gray-1 width height r g b)
+  (let ((n-cell (* width height))
+        (to (im-make-channel width height)))
+    (f32vector-rgb-to-gray to n-cell r g b)
+    to))
+
+#;(define (im-rgb->gray-1 width height r g b)
+  (letrec* ((rgb->gray (lambda (i)
+                         (/ (+ (f32vector-ref r i)
+                               (f32vector-ref g i)
+                               (f32vector-ref b i))
+                            3)))
+            (to (im-make-channel width height))
+            (n-cell (* width height))
+            (proc (lambda (range)
+                    (match range
+                      ((start end)
+                       (let ((k0 (rgb->gray start)))
+                         (let loop ((i (+ start 1))
+                                    (mini k0)
+                                    (maxi k0)
+                                    (total k0))
+                           (if (= i end)
+                               (list mini maxi total)
+                               (let ((k (rgb->gray i)))
+                                 (f32vector-set! to i k)
+                                 (loop (+ i 1)
+                                       (min mini k)
+                                       (max maxi k)
+                                       (+ total k)))))))))))
+    (if (%use-par-map)
+        (let ((vals (par-map proc
+                         (n-cell->per-core-start-end n-cell))))
+          (values to
+                  (list (apply min (map car vals))
+                        (apply max (map cadr vals))
+                        (/ (reduce + 0 (map caddr vals)) n-cell)
+                        n-cell)))
+        (match (proc (list 0 n-cell))
+          ((mini maxi total)
+           (values to
+                   (list mini
+                         maxi
+                         (/ total n-cell)
+                         n-cell)))))))
+
+#;(define (im-rgb->gray-1 width height r g b)
+  (letrec* ((rgb->gray (lambda (i)
+                         (/ (+ (f32vector-ref r i)
+                               (f32vector-ref g i)
+                               (f32vector-ref b i))
+                            3)))
+            (n-cell (* width height))
+            (to (make-f32vector n-cell 0.0))
+            (k0 (rgb->gray 0))
+            (t0 (f32vector-set! to 0 k0)))
+    (let loop ((i 1)
+               (mini k0)
+               (maxi k0)
+               (total k0))
+      (if (= i n-cell)
+          (values to
+                  mini
+                  maxi
+                  (/ total n-cell))
+          (let ((k (rgb->gray i)))
+            (f32vector-set! to i k)
+            (loop (+ i 1)
+                  (min mini k)
+                  (max maxi k)
+                  (+ total k)))))))
+
 #!
 Source => Target = (BGColor + Source) =
 Target.R = ((1 - Source.A) * BGColor.R) + (Source.A * Source.R)
@@ -178,32 +250,59 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
        (else
 	(error "Not an RGBA (nor an RGB neither a GRAY) image."))))))
 
-(define* (im-threshold image threshold #:key (bg 'black) (prec 1.0e-4))
+(define* (im-threshold image threshold #:key (bg 'black))
   (if (and (>= threshold 0.0)
 	   (<= threshold 255.0))
-      (match (match image
-               ((_ _ n-chan _)
-                (case n-chan
-                  ((1) image)
-                  ((3) (im-rgb->gray image))
-                  ((4) (im-rgba->gray image))
-                  (else
-                   (error "Not a GRAY, RGB, nor an RGBA image.")))))
+      (match image
 	((width height n-chan idata)
-	 (match idata
-	   ((c)
-	    (let ((c-copy (im-make-channel width height))
-		  (op (case bg
-			((black) float>=?)
-			((white) float<=?)
-			(else
-			 (error "Invalid background: " bg)))))
-	      (do ((i 0
-		      (+ i 1)))
-		  ((= i (* width height))
-		   (list width height 1 (list c-copy)))
-		(when (op (f32vector-ref c i) threshold prec)
-		  (f32vector-set! c-copy i 255.0))))))))
+         (letrec* ((test (case bg
+                           ((black) >=)
+                           ((white) <=)
+                           (else
+                            (error "Invalid background: " bg))))
+                   (pred (match idata
+                           ((r g b)
+                            (lambda (i threshold)
+                              (test (/ (+ (f32vector-ref r i)
+                                          (f32vector-ref g i)
+                                          (f32vector-ref b i))
+                                       3)
+                                    threshold)))
+                           ((c)
+                            (lambda (i threshold)
+                              (test (f32vector-ref c i) threshold)))
+                           (else
+                            (error "Not a GRAY, RGB, nor an RGBA image."))))
+                   (to (im-make-channel width height))
+                   (n-cell (* width height))
+                   (proc (lambda (range)
+                           (match range
+                             ((start end)
+                              (do ((i start
+                                      (+ i 1)))
+                                  ((= i end))
+                                (when (pred i threshold)
+                                  (f32vector-set! to i 255.0))))))))
+           (if (%use-par-map)
+               (par-for-each proc (n-cell->per-core-start-end n-cell))
+               (proc (list 0 n-cell)))
+           (list width height 1 (list to)))))
+      (error "Invalid threshold: " threshold)))
+
+(define* (im-threshold image threshold #:key (bg 'black))
+  (if (and (>= threshold 0.0)
+	   (<= threshold 255.0))
+        (match image
+          ((width height n-chan idata)
+           (let ((to (im-make-channel width height))
+                 (n-cell (* width height)))
+             (list width height 1
+                   (list (f32vector-threshold to n-cell idata threshold
+                                              (case bg
+                                                ((black) 0)
+                                                ((white) 255)
+                                                (else
+                                                 (error "No such bg: " bg)))))))))
       (error "Invalid threshold: " threshold)))
 
 (define (im-matrix-op image img-2 op)
@@ -259,39 +358,102 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                               (zip idata-1 idata-2))))
             (error "Size missmatch.")))))))
 
-
-(define-method (im-add image (val <number>))
+#;(define-method (im-add image (val <number>))
   (im-map (lambda (p-val) (+ p-val val)) image))
 
-(define-method (im-add-channel channel width height (val <number>))
+#;(define-method (im-add-channel channel width height (val <number>))
   (im-map-channel (lambda (p-val) (+ p-val val)) width height channel))
 
-(define-method (im-add . images)
+(define-method (im-add image (val <number>))
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+             (map-proc (lambda (channel)
+                         (im-add-channel channel width height val))
+                 idata))))))
+
+(define-method (im-add-channel channel width height (val <number>))
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (f32vector-add-value channel val to #:n-cell n-cell)
+    to))
+
+#;(define-method (im-add . images)
   (apply im-map + images))
 
-(define-method (im-add-channel width height . channels)
+#;(define-method (im-add-channel width height . channels)
   (apply im-map-channel + width height channels))
 
+(define-method (im-add . images)
+  (im-map-la im-add-channel images))
 
-(define-method (im-subtract image (val <number>))
+(define-method (im-add-channel width height . channels)
+  (let ((n-cell (* width height))
+        (to (im-make-channel width height)))
+    (f32vector-add-vectors to n-cell channels)))
+
+
+#;(define-method (im-subtract image (val <number>))
   (im-map (lambda (p-val) (- p-val val)) image))
 
-(define-method (im-subtract-channel channel width height (val <number>))
+#;(define-method (im-subtract-channel channel width height (val <number>))
   (im-map-channel (lambda (p-val) (- p-val val)) width height channel))
 
-(define-method (im-subtract . images)
+(define-method (im-subtract image (val <number>))
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+             (map-proc (lambda (channel)
+                         (im-subtract-channel channel width height val))
+                 idata))))))
+
+(define-method (im-subtract-channel channel width height (val <number>))
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (f32vector-subtract-value channel val to #:n-cell n-cell)
+    to))
+
+#;(define-method (im-subtract . images)
   (apply im-map - images))
 
-(define-method (im-subtract-channel width height . channels)
+#;(define-method (im-subtract-channel width height . channels)
   (apply im-map-channel - width height channels))
 
+(define-method (im-subtract . images)
+  (im-map-la im-subtract-channel images))
 
-(define-method (im-multiply image (val <number>))
+(define-method (im-subtract-channel width height . channels)
+  (let ((n-cell (* width height))
+        (to (im-make-channel width height)))
+    (f32vector-subtract-vectors to n-cell channels)))
+
+
+#;(define-method (im-multiply image (val <number>))
   (im-map (lambda (p-val) (* p-val val)) image))
 
-(define-method (im-multiply-channel channel width height (val <number>))
+#;(define-method (im-multiply-channel channel width height (val <number>))
   (im-map-channel (lambda (p-val) (* p-val val)) width height channel))
 
+(define-method (im-multiply image (val <number>))
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+             (map-proc (lambda (channel)
+                         (im-multiply-channel channel width height val))
+                 idata))))))
+
+(define-method (im-multiply-channel channel width height (val <number>))
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (f32vector-multiply-value channel val to #:n-cell n-cell)
+    to))
+
 (define (im-multiply-1 prev images)
   (if (null? images)
       prev
@@ -324,12 +486,31 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
      (error "Wrong arguments:" rest))))
 
 
-(define-method (im-divide image (val <number>))
+#;(define-method (im-divide image (val <number>))
   (im-map (lambda (p-val) (/ p-val val)) image))
 
-(define-method (im-divide-channel channel width height (val <number>))
+#;(define-method (im-divide-channel channel width height (val <number>))
   (im-map-channel (lambda (p-val) (/ p-val val)) width height channel))
 
+(define-method (im-divide image (val <number>))
+  (if (= val 0.0)
+      (error "Attempt to divide by 0")
+      (match image
+        ((width height n-chan idata)
+         (list width height n-chan
+               (let ((map-proc (if (and (> n-chan 1)
+                                        (%use-par-map)) par-map map)))
+                 (map-proc (lambda (channel)
+                             (im-divide-channel channel width height val))
+                     idata)))))))
+
+(define-method (im-divide-channel channel width height (val <number>))
+  (if (= val 0.0)
+      (error "Attempt to divide by 0")
+      (let ((to (im-make-channel width height))
+            (n-cell (* width height)))
+        (f32vector-divide-value channel val to #:n-cell n-cell)
+        to)))
 
 (define (im-divide-1 prev images)
   (if (null? images)
@@ -364,6 +545,36 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
      (error "Wrong arguments:" rest))))
 
 
+(define-method (im-map-la im-map-la-channel image (val <number>))
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+             (map-proc (lambda (channel)
+                         (apply im-map-la-channel width height channel val))
+                 idata))))))
+
+(define-method (im-map-la im-map-la-channel images)
+  (match images
+    ((image) image)
+    ((image . rest)
+     (match image
+       ((width height n-chan _)
+	(if (and (apply = (apply im-collect 'width rest))
+		  (apply = (apply im-collect 'height rest))
+		  (apply = (apply im-collect 'n-channel rest)))
+            (list width height n-chan
+                  (let ((map-proc (if (and (> n-chan 1)
+                                           (%use-par-map)) par-map map)))
+                    (map-proc (lambda (channels)
+                                (apply im-map-la-channel width height channels))
+                        (apply zip (apply im-collect 'channels images)))))
+	    (error "Size mismatch.")))))
+    (()
+     (error "Invalid argument: " images))))
+
+
 (define (im-range image)
   (match image
     ((width height n-chan idata)
@@ -437,6 +648,15 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
 
 (define (im-map proc . images)
   (match images
+    ((image)
+     (match image
+       ((width height n-chan idata)
+        (list width height n-chan
+              (let ((map-proc (if (and (> n-chan 1)
+                                       (%use-par-map)) par-map map)))
+                (map-proc (lambda (channel)
+                            (im-map-channel proc width height channel))
+                    idata))))))
     ((image . rest)
      (match image
        ((width height n-chan _)
@@ -450,15 +670,6 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                                 (apply im-map-channel proc width height channels))
                         (apply zip (apply im-collect 'channels images)))))
 	    (error "Size mismatch.")))))
-    ((image)
-     (match image
-       ((width height n-chan idata)
-        (list width height n-chan
-              (let ((map-proc (if (and (> n-chan 1)
-                                       (%use-par-map)) par-map map)))
-                (map-proc (lambda (channel)
-                            (im-map-channel proc width height channel))
-                    idata))))))
     (()
      (error "Invalid argument: " images))))
 
@@ -471,8 +682,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                 (+ i 1)))
             ((= i n-cell))
           (f32vector-set! to i
-                          (apply proc
-                                 (f32vector-ref-at-offset channels i))))
+                          (f32vector-reduce-at-offset channels proc 0.0 i)))
         (do ((c (car channels))
              (i 0
                 (+ i 1)))
@@ -481,7 +691,8 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                             (proc (f32vector-ref c i)))))
     to))
 
-(define (im-and . images)
+
+#;(define (im-and . images)
   (match images
     ((image . rest)
      (match image
@@ -507,7 +718,43 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
     ((image) image)
     (() (error "Invalid argument: " images))))
 
-(define (im-or . images)
+(define (im-and . images)
+  (match images
+    ((image) image)
+    ((i1 . rest)
+     (match i1
+       ((width height n-chan idata)
+	(if (and (apply = (apply im-collect 'width images))
+                 (apply = (apply im-collect 'height images)))
+            (let ((c-rest (apply im-collect 'gray (map im-rgb->gray rest))))
+              (match idata
+                ((c1 . rest)
+                 (let ((c-and (apply im-and-channel width height c-rest)))
+                   (list width height n-chan
+                         (let ((map-proc (if (and (> n-chan 1)
+                                                  (%use-par-map)) par-map map)))
+                           (map-proc (lambda (chan)
+                                       (im-and-channel width height chan c-and))
+                               idata)))))
+                ((c)
+                 (list width height n-chan
+                       (apply im-and-channel width height
+                              (cons c c-rest))))))
+	    (error "Size mismatch.")))))
+    (() (error "Invalid argument: " images))))
+
+(define (im-and-channel width height . channels)
+  (match channels
+    ((c1) c1)
+    ((c1 . rest)
+     (let ((n-cell (* width height))
+           (to (im-make-channel width height)))
+       (f32vector-and-vectors to n-cell channels)))
+    (else
+     (error "Invalid argument:" channels))))
+
+
+#;(define (im-or . images)
   (match images
     ((image . rest)
      (match image
@@ -534,7 +781,43 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
     ((image) image)
     (() (error "Invalid argument: " images))))
 
-(define (im-xor . images)
+(define (im-or . images)
+  (match images
+    ((image) image)
+    ((i1 . rest)
+     (match i1
+       ((width height n-chan idata)
+	(if (and (apply = (apply im-collect 'width images))
+                 (apply = (apply im-collect 'height images)))
+            (let ((c-rest (apply im-collect 'gray (map im-rgb->gray rest))))
+              (match idata
+                ((c1 . rest)
+                 (let ((c-or (apply im-or-channel width height c-rest)))
+                   (list width height n-chan
+                         (let ((map-proc (if (and (> n-chan 1)
+                                                  (%use-par-map)) par-map map)))
+                           (map-proc (lambda (chan)
+                                       (im-or-channel width height chan c-or))
+                               idata)))))
+                ((c)
+                 (list width height n-chan
+                       (apply im-or-channel width height
+                              (cons c c-rest))))))
+	    (error "Size mismatch.")))))
+    (() (error "Invalid argument: " images))))
+
+(define (im-or-channel width height . channels)
+  (match channels
+    ((c1) c1)
+    ((c1 . rest)
+     (let ((n-cell (* width height))
+           (to (im-make-channel width height)))
+       (f32vector-or-vectors to n-cell channels)))
+    (else
+     (error "Invalid argument:" channels))))
+
+
+#;(define (im-xor . images)
   (match images
     ((image . rest)
      (match image
@@ -552,7 +835,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
     ((image) image)
     (() (error "Invalid argument: " images))))
 
-(define (im-xor-channels channels width height)
+#;(define (im-xor-channels channels width height)
   (let ((to (im-make-channel width height))
         (n-cell (* width height)))
     (do ((i 0
@@ -562,6 +845,35 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                       (f32vector-xor-at-offset channels i)))
     to))
 
+(define (im-xor . images)
+  (match images
+    ((image) image)
+    ((i1 . rest)
+     (match i1
+       ((width height n-chan idata)
+	(if (and (apply = (apply im-collect 'width images))
+                 (apply = (apply im-collect 'height images))
+                 (apply = (apply im-collect 'n-channel rest)))
+            (list width height n-chan
+                  (let ((map-proc (if (and (> n-chan 1)
+                                           (%use-par-map)) par-map map)))
+                    (map-proc (lambda (channels)
+                                (apply im-xor-channel width height channels))
+                        (apply zip (apply im-collect 'channels images)))))
+	    (error "Size mismatch.")))))
+    (() (error "Invalid argument: " images))))
+
+(define (im-xor-channel width height . channels)
+  (match channels
+    ((c1) c1)
+    ((c1 . rest)
+     (let ((n-cell (* width height))
+           (to (im-make-channel width height)))
+       (f32vector-xor-vectors to n-cell channels)))
+    (else
+     (error "Invalid argument:" channels))))
+
+
 (define (im-complement image)
   (match image
     ((width height n-chan idata)
@@ -629,7 +941,7 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                               val)))
     to))
 
-(define* (im-scrap image val #:key (pred <) (con 8) (bg 'black))
+(define* (im-scrap image val #:key (pred <) (con 8) (bg 'black) (exclude-on-edges #f))
   ;; (im-binary? image) is rather expensive
   (match image
     ((width height n-chan idata)
@@ -646,8 +958,14 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                        (n-feature (length features))
                        (to-scrap (fold (lambda (feature i prev)
                                          (match feature
-                                           ((area . rest)
-                                            (if (pred area val)
+                                           ((area left top right bottom . rest)
+                                            (if (and (not (= i 0))
+                                                     (or (pred area val)
+                                                         (and exclude-on-edges
+                                                              (or (= left 0)
+                                                                  (= top 0)
+                                                                  (= right (- width 1))
+                                                                  (= bottom (- height 1))))))
                                                 (cons i prev)
                                                 prev))))
                                        '()
@@ -659,34 +977,144 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
        (else
         (error "Not a binary image."))))))
 
-#!
-;; nice but too slow
-(define (im-scrap-channel channel l-channel width height to-scrap)
-  (let ((to (im-copy-channel channel width height))
-        (n-cell (* width height))
-        (to-scrap (list->vector (reverse! to-scrap))))
+(define (scrap-cache to-scrap n-label)
+  (let ((n-scrap (length to-scrap))
+        (cache (make-s32vector n-label 0)))
     (do ((i 0
-	    (+ i 1)))
-	((= i n-cell))
-      (let ((val (f32vector-ref l-channel i)))
-        (f32vector-set! to i
-                        (if (or (= val 0.0)
-                                (binary-search-sorted-vector to-scrap val))
-                            0.0
-                            (f32vector-ref channel i)))))
+            (+ i 1)))
+	((= i n-scrap))
+      (s32vector-set! cache (list-ref to-scrap i) 1))
+    cache))
+
+(define (im-scrap-channel channel l-channel width height to-scrap n-label)
+  (let* ((to (im-copy-channel channel width height))
+         (n-cell (* width height))
+         (cache (scrap-cache to-scrap n-label)))
+    (f32vector-scrap channel l-channel n-cell cache to)
     to))
-!#
+
+(define* (im-particles image features #:key (clean #t))
+  (let* ((map-proc (if (%use-par-map) par-map map))
+         (particles (map-proc
+                        (lambda (prop)
+                          (match prop
+                            ((area left top right bottom . rest)
+                             (parameterize ((%use-par-map #f))
+                               (let ((particle (im-crop image
+                                                        left top (+ right 1) (+ bottom 1))))
+                                 (list (if clean
+                                           (im-particle-clean particle)
+                                           particle)
+                                       (list left top right bottom)))))))
+                        (cdr features))))
+    (values (map car particles)
+            (map cadr particles))))
+
+(define* (im-particle-clean particle #:key (binary? #t))
+  (let* ((p-bin (if binary? particle (im-threshold particle 1.0)))
+         (p-clean (match p-bin
+                    ((width height n-chan idata)
+                     (receive (p-label n-label)
+                         (im-label p-bin)
+                       (let* ((n-object (- n-label 1))
+                              (r (- width 1))
+                              (b (- height 1))
+                              (p-bin-chan (im-channel p-bin 0))
+                              (p-label-chan (im-channel p-label 0))
+                              (n-cell (* width height))
+                              (to-scrap (fold (lambda (prop i prev)
+                                                (match prop
+                                                  ((_ left top right bottom . rest)
+                                                   (if (or (not (= left 0))
+                                                           (not (= top 0))
+                                                           (not (= right r))
+                                                           (not (= bottom b)))
+                                                       ;; we did skip the bg, so i needs 1+
+                                                       (cons (+ i 1) prev)
+                                                       prev))))
+                                              '()
+                                              (cdr (im-features p-bin p-label
+                                                                #:n-label n-label))
+                                              (iota n-object))))
+                         (list width height n-chan
+                               (list (match to-scrap
+                                       (() p-bin-chan)
+                                       (else
+                                        (f32vector-scrap-in-place p-bin-chan
+                                                                  p-label-chan
+                                                                  n-cell
+                                                                  (scrap-cache to-scrap n-label))
+                                        p-bin-chan
+                                        #;(let ((cache (scrap-cache to-scrap n-label)))
+                                          (do ((i 0
+                                                  (+ i 1)))
+                                              ((= i n-cell) p-bin-chan)
+                                            (let ((val
+                                                   (float->int (f32vector-ref p-label-chan i))))
+                                              ;; (when (vector-ref cache val)
+                                              (unless (= (s32vector-ref cache val) 0)
+                                                (f32vector-set! p-bin-chan i 0.0)))))))))))))))
+    (if binary?
+        p-clean
+        (im-and particle p-clean))))
+
+(define (%merge-channel-color i)
+  (case (i)
+    ((3) '(255.0 255.0 255.0))	;; white
+    ((4) '(0.0 255.0 255.0))	;; cyan
+    ((5) '(255.0 0.0 255.0))	;; magenta
+    ((6) '(255.0 255.0 0.0))	;; yellow
+    (else
+     "No such merge channel color index:" i)))
+
+(define (im-composite->rgb image)
+  (match image
+    ((width height n-chan idata)
+     (match idata
+       ((c1 c2 c3 c4 . rest)
+        (let ((n-cell (* width height))
+              (r-chan (im-make-channel width height))
+              (g-chan (im-make-channel width height))
+              (b-chan (im-make-channel width height)))
+          (do ((i 0
+                  (+ i 1)))
+              ((= i n-cell)
+               (list width height 3
+                     (list r-chan g-chan b-chan)))
+            (if (= (f32vector-ref c4 i) 255.0)
+                (begin
+                  (f32vector-set! r-chan i 255.0)
+                  (f32vector-set! g-chan i 255.0)
+                  (f32vector-set! b-chan i 255.0))
+                (begin
+                  (f32vector-set! r-chan i
+                                  (f32vector-ref c1 i))
+                  (f32vector-set! g-chan i
+                                  (f32vector-ref c2 i))
+                  (f32vector-set! b-chan i
+                                  (f32vector-ref c3 i)))))))
+       (else
+        (error "Not a composite image: " image))))))
+
+
+#!
+
+;; ok for small images, but too slow otherwise so, till Guile-3.0, I
+;; have to do this in C instead which is fine anyway, because memory is
+;; allocated on the scheme side. Let's keep these for now, once
+;; Guile-3.0 is out, it will be nice to try and compare with those
+;; versions hsing libgule-cv.
 
 (define (scrap-cache to-scrap n-label)
   (let ((n-scrap (length to-scrap))
-         (cache (make-vector (+ n-label 1) #f)))
+        (cache (make-vector n-label #f)))
     (do ((i 0
             (+ i 1)))
 	((= i n-scrap))
       (vector-set! cache (list-ref to-scrap i) #t))
     cache))
 
-(define* (im-scrap-channel channel l-channel width height to-scrap n-label)
+(define (im-scrap-channel channel l-channel width height to-scrap n-label)
   (let* ((to (im-copy-channel channel width height))
          (n-cell (* width height))
          (cache (scrap-cache to-scrap n-label)))
@@ -702,49 +1130,4 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
                             (f32vector-ref channel i)))))
     to))
 
-(define* (im-particles image features #:key (clean #t))
-  (let ((map-proc (if (%use-par-map) par-map map)))
-    (map-proc (lambda (prop)
-                (match prop
-                  ((area left top right bottom . rest)
-                   (parameterize ((%use-par-map #f))
-                     (let ((particle (im-crop image left top (+ right 1) (+ bottom 1))))
-                       (if clean
-                           (im-particle-clean particle)
-                           particle))))))
-              (cdr features))))
-
-(define (im-particle-clean particle)
-  (let* ((binary? (im-binary? particle))
-         (binary (if binary? particle (im-threshold particle 1.0)))
-         (cleaned (match binary
-                    ((width height n-chan idata)
-                     (receive (label-im n-label)
-                         (im-label binary)
-                       (let* ((r (- width 1))
-                              (b (- height 1))
-                              (l-channel (im-channel label-im 0))
-                              (n-cel (* width height))
-                              (to-remove (fold (lambda (prop i prev)
-                                                 (match prop
-                                                   ((size left top right bottom . rest)
-                                                    (if (or (not (= left 0))
-                                                            (not (= top 0))
-                                                            (not (= right r))
-                                                            (not (= bottom b)))
-                                                        (cons i prev)
-                                                        prev))))
-                                               '()
-                                               (im-features binary label-im
-                                                            #:n-label n-label)
-                                               (iota (+ n-label 1))))
-                              (cache (scrap-cache to-remove n-label)))
-                         (do ((i 0
-                                 (+ i 1)))
-                             ((= i n-cel)
-                              (list width height n-chan (list l-channel)))
-                           (let ((val
-                                  (float->int (f32vector-ref l-channel i))))
-                             (when (vector-ref cache val)
-                               (f32vector-set! l-channel i 0.0))))))))))
-    (if binary? cleaned (im-and particle cleaned))))
+!#
